Title: Clearly I am the Worst Autotools User
Author: Kendall E. Blake
Date: 2019-11-16 20:02
Category: Programming
Tags: autotools, flamegraph, u3

I've been trying to determine why my game was running slowly after a couple of
changes went in: flickering, glowing light sources (drawn pixel-at-a-time so
that I can avoid 3D, because I like slooooow) and some new agent routines.
Behavioral routines for the agents, that is, not just _code_ routines.  This
slowdown happened when it get dark, but that's when both is the performance
regression due to one of these changes?  How can hunt down the issue?

The answer to the first question: of course, and it was both things, huzzah.

The answer to the second question is the fun part, specifically the part
wherein I learn a new thing about Autotools and realize that, again, I am the
worst user ever.

I've written a very first-pass at an A-star pathfinder for my agents.  It works
well enough (yay) and I know that it's very slow.  And it's not that I wrote it
in Python.  I think that part's actually okay.  The issue is that I'm calling
into the c++ half of the code (Thanks Boost!) and handing the data back
piecemeal rather than extracting the pertinent information out in bulk from
the c++ code, and move it out of the loop(s).

I've done some little things like decrease the maximum area to search to see
how that affected the perceived choppiness of the game.  It reduced the effect
somewhat, but not consistently.  When agents had all finished their movement
routine and stopped pathfinding, the degredation was still apparent.  I looked
at the code a bit more and started doing some "printf debugging" and thought,
as I always do when a "printf debug" doesn't immediatley reveal the issue:
what's the right way to look at the hotspots and what code is actually being
run?

[FlameGraph](http://www.brendangregg.com/flamegraphs.html)

So, I do my quick brush up on the "how many perf invocations do I have to do?"
All the while thinking to myself, "I've run perf like 5 times in my life. I
hope I don't have to learn how to use this.  I'm like, you know.  The _worst_
user.

Perf is easy, just:
```
  perf record -e cycles -g --call-graph fp -- $commandline --goes --here
```

Note that I've got a crazy invocation.  I was trying to find out what worked,
ran that in an abortive test and didn't roll back to the starting point.  I
break things, it is fun.  When you break things, you get to fix them!  It's
like a present.  I suspect that:
```
  perf record -g -- $commandline --goes --here
```

will work just as well, right?  Right.  (I haven't tested.  If I did, I'd be
playing a game instead of working.  I really don't want to have to fire myself.
Seems awkward, dontchathink?)

It does, those are exactly the correct invocations to record a process
started by that `$commandline` bit.

Next step is to uh, write out the script?  With
`perf script > $scriptfile  #  Idk, my bff jill` [1].
It and `perf record` have a shared file naming convention.  Isn't that nice?
It's almost as though they were the same program.  I kid, I kid.  I'm sure it
makes sense, but, well, you know.  `perf record` will output to `out.perf` and
`perf script` will read from `out.perf`.  `perf script` then outputs to
`stdout`, so it's probably all text.  (Remember Jill.)

We are doing so well!  We're at the final step now (more huzzah.)

So, we're into fun with file descriptors, it's the unix way!  G'head and
copy-paste this one.  I know I will.  [george takei here]
```
stackcollapse-perf.pl < perf.script | flamegraph.pl > out.svg
```

(Aside: Fedora has FlameGraph packaged so I've got it `dnf install`ed.  The
upstream docs assume that you are running it from the current directory.)

And open in your friendly neigborhood
[Web Browser] (https://www.seamonkey-project.org)!  Just kidding, I believe in
browser choice, you can, actually, maybe you _can't_ open it in IE6.

This brief aside has been brought to you by Future Me, who is, right now
reading these words and smiling back into the past.  This will save him a
couple of hours.

* * *

So here we are at autotools.  I've been using autotools to build software since
the 1990s.  It's the only build system I know and, ah, let me rephrase that.
The only system that I'm capable of pretending to know.  Apparently part of
that pretense is not knowing that "no `$C{,XX}FLAGS`" means
`-O2 --fomit-frame pointer`."  No frame pointers means no way for perf to tell
you which function you're in.  And your flame graph is full of `[unknown]`.
Oh, perf, if only you knew.

The saddest part is that I, most of the time, just pass `CFLAGS` and `CXXFLAGS`
into my `configure` invoication and have done with.  This is after wasting
inordinate amounts of past Kendall's time with tomfoolery.  I've learned this
already.  But not as a "thou shalt" and "only do it this way."  Well, "thou
shalt" and only ever define your `CFLAGS` and `CXXFLAGS` in the invoking
environment.

Also your `config.site` file.  Which is totally the way to go.  If I were
in the business of packaging code from upstream, I'd see how close to "just
putting `config.site` under revision control" that I could manage.  Mmmm.
Future me: I bet someone's already on this one!

[1]: Always comment your in-paragraph code snippets. Otherwise, when the reader
invariably copies-and-pastes it, hopefully the comment will, and it will be
a sign that the blind are leading the blind.
